﻿using DressAtelierContracts.BindingModels;
using DressAtelierContracts.BusinessLogicContracts;
using DressAtelierContracts.SearchModels;
using DressAtelierContracts.StorageContracts;
using DressAtelierContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DressAtelierBusinessLogic.BusinessLogic
{
    public class ClientLogic : IClientLogic
    {
        private readonly ILogger _logger;
        private readonly IClientStorage _clientStorage;
        public ClientLogic(ILogger<ClientLogic> logger, IClientStorage clientStorage)
        {
            _logger = logger;
            _clientStorage = clientStorage;
        }
        public bool Create(ClientBindingModel model)
        {
            CheckUser(model);
            if(_clientStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ClientBindingModel model)
        {
            CheckUser(model);
            if (_clientStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ClientBindingModel model)
        {
            CheckUser(model,false);
            _logger.LogInformation("Delete. ID:{ID}", model.ID);
            if (_clientStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public ClientViewModel? ReadElement(ClientSearchModel model)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. UserName:{UserName}.ID:{ ID}", model.Email, model.ID);
            var element = _clientStorage.GetElement(model);
            if(element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. ID:{ID}", element.ID);
            return element;
        }

        public List<ClientViewModel>? ReadList(ClientSearchModel? model)
        {
            _logger.LogInformation("ReadList. UserName:{UserName}. ID:{ ID}", model?.Email, model?.ID);

            var list = model == null ? _clientStorage.GetFullList() : _clientStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public void CheckUser(ClientBindingModel model,bool withParams = true)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if(!withParams)
            {
                return;
            }
            if(string.IsNullOrEmpty(model.FullName))
            {
                throw new ArgumentNullException("Invalid fullname of user", nameof(model.FullName));
            }
            if (string.IsNullOrEmpty(model.Email))
            {
                throw new ArgumentNullException("Invalid email of user", nameof(model.Email));
            }
            if (string.IsNullOrEmpty(model.Password))
            {
                throw new ArgumentNullException("Invalid password of user", nameof(model.Password));
            }
            _logger.LogInformation("Client. ClientName:{ FullName}. Email:{ Email}. ID: { ID} ", model.FullName, model.Email, model.ID);

            var element = _clientStorage.GetElement(new ClientSearchModel
            {
                Email = model.Email
            });

            if(element != null && element.ID != model.ID)
            {
                throw new InvalidOperationException("User with such email already exists.");
            }
        }
       
    }
}
